<?php

declare(strict_types=1);

/*
 * Chill is a software for social workers
 *
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace Chill\TicketBundle\Action\Ticket\Handler;

use Chill\MainBundle\Phonenumber\PhonenumberHelper;
use Chill\PersonBundle\Repository\PersonACLAwareRepositoryInterface;
use Chill\TicketBundle\Action\Ticket\AssociateByPhonenumberCommand;
use Chill\TicketBundle\Entity\PersonHistory;
use Chill\TicketBundle\Entity\Ticket;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\Clock\ClockInterface;

class AssociateByPhonenumberCommandHandler
{
    public function __construct(
        private PersonACLAwareRepositoryInterface $personRepository,
        private PhonenumberHelper $phonenumberHelper,
        private ClockInterface $clock,
        private EntityManagerInterface $entityManager,
    ) {}

    public function __invoke(Ticket $ticket, AssociateByPhonenumberCommand $command): void
    {
        $phone = $this->phonenumberHelper->parse($command->phonenumber);
        $persons = $this->personRepository->findByPhone($phone);

        foreach ($persons as $person) {
            $history = new PersonHistory($person, $ticket, $this->clock->now());
            $this->entityManager->persist($history);
        }
    }
}
