<?php

declare(strict_types=1);

/*
 * Chill is a software for social workers
 *
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace Chill\TicketBundle\Action\Ticket\Handler;

use Chill\TicketBundle\Action\Ticket\ReplaceMotiveCommand;
use Chill\TicketBundle\Entity\MotiveHistory;
use Chill\TicketBundle\Entity\Ticket;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\Clock\ClockInterface;

final readonly class ReplaceMotiveCommandHandler
{
    public function __construct(
        private ClockInterface $clock,
        private EntityManagerInterface $entityManager,
    ) {}

    public function handle(Ticket $ticket, ReplaceMotiveCommand $command): void
    {
        if (null === $command->motive) {
            throw new \InvalidArgumentException('The new motive cannot be null');
        }

        // will add if there are no existing motive
        $readyToAdd = 0 === count($ticket->getMotiveHistories());

        foreach ($ticket->getMotiveHistories() as $history) {
            if (null !== $history->getEndDate()) {
                continue;
            }

            if ($history->getMotive() === $command->motive) {
                // we apply the same motive, we do nothing
                continue;
            }

            $history->setEndDate($this->clock->now());
            $readyToAdd = true;
        }

        if ($readyToAdd) {
            $history = new MotiveHistory($command->motive, $ticket, $this->clock->now());
            $this->entityManager->persist($history);
        }
    }
}
