<?php

declare(strict_types=1);

/*
 * Chill is a software for social workers
 *
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace Chill\TicketBundle\Controller;

use Chill\MainBundle\Phonenumber\PhonenumberHelper;
use Chill\PersonBundle\Repository\PersonRepository;
use Chill\PersonBundle\Templating\Entity\PersonRenderInterface;
use libphonenumber\NumberParseException;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Symfony\Component\Routing\Annotation\Route;

/**
 * Controller for a rest api to find a caller for a given phonenumber.
 *
 * TODO: currently, this rest api is not secured
 */
class FindCallerController
{
    public function __construct(private PhonenumberHelper $phonenumberHelper, private PersonRepository $personRepository, private PersonRenderInterface $personRender) {}

    #[Route('/public/api/1.0/ticket/find-caller', name: 'find-caller', methods: ['GET'])]
    public function findCaller(Request $request): Response
    {
        $caller = $request->query->get('caller', '');

        if ('' === $caller) {
            throw new BadRequestHttpException('Missing "caller" query parameter');
        }

        try {
            $phoneNumber = $this->phonenumberHelper->parse($caller);
        } catch (NumberParseException $e) {
            throw new BadRequestHttpException('Unable to parse number', $e);
        }

        $persons = $this->personRepository->findByPhoneNumber($phoneNumber, 0, 2);

        $asArray = match (count($persons)) {
            0 => ['found' => false, 'name' => null],
            1 => ['found' => true, 'name' => $this->personRender->renderString($persons[0], ['addAge' => false])],
            default => ['found' => true, 'name' => 'multiple'],
        };

        return new JsonResponse($asArray);
    }
}
