<?php

declare(strict_types=1);

/*
 * Chill is a software for social workers
 *
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace Chill\TicketBundle\Controller;

use Chill\MainBundle\Templating\Listing\FilterOrderHelper;
use Chill\MainBundle\Templating\Listing\FilterOrderHelperFactory;
use Chill\MainBundle\Templating\TranslatableStringHelperInterface;
use Chill\TicketBundle\Entity\Motive;
use Chill\TicketBundle\Repository\MotiveRepository;
use Chill\TicketBundle\Repository\TicketRepositoryInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\Security\Core\Security;
use Twig\Environment;
use Twig\Error\LoaderError;
use Twig\Error\RuntimeError;
use Twig\Error\SyntaxError;

final readonly class TicketListController
{
    public function __construct(
        private Security $security,
        private TicketRepositoryInterface $ticketRepository,
        private Environment $twig,
        private FilterOrderHelperFactory $filterOrderHelperFactory,
        // private MotiveRepository $motiveRepository,
        // private TranslatableStringHelperInterface $translatableStringHelper,
    ) {}

    /**
     * @throws RuntimeError
     * @throws SyntaxError
     * @throws LoaderError
     */
    #[Route('/{_locale}/ticket/ticket/list', name: 'chill_ticket_ticket_list')]
    public function __invoke(Request $request): Response
    {
        if (!$this->security->isGranted('ROLE_USER')) {
            throw new AccessDeniedHttpException('only user can access this page');
        }

        $filter = $this->buildFilter();

        $tickets = $this->ticketRepository->findAllOrdered();

        return new Response(
            $this->twig->render('@ChillTicket/Ticket/list.html.twig', [
                'tickets' => $tickets,
                'filter' => $filter,
            ])
        );
    }

    private function buildFilter(): FilterOrderHelper
    {
        // $motives = $this->motiveRepository->findAll();

        return $this->filterOrderHelperFactory
            ->create(__CLASS__)
            ->addSingleCheckbox('to_me', 'chill_ticket.list.filter.to_me')
            ->addSingleCheckbox('in_alert', 'chill_ticket.list.filter.in_alert')
            ->addDateRange('created_between', 'chill_ticket.list.filter.created_between')
            /*
            ->addEntityChoice('by_motive', 'chill_ticket.list.filter.by_motive', Motive::class, $motives, [
                'choice_label' => fn (Motive $motive) => $this->translatableStringHelper->localize($motive->getLabel()),
                'expanded' => true,
                'multiple' => true,
                'attr' => ['class' => 'select2'],
            ])
            */
            ->build();
    }
}
