import {
    DateTime,
    TranslatableString,
    User,
    UserGroupOrUser,
} from "../../../../ChillMainBundle/Resources/public/types";
import { Person } from "../../../../ChillPersonBundle/Resources/public/types";

export interface Motive {
    type: "ticket_motive";
    id: number;
    active: boolean;
    label: TranslatableString;
}

interface TicketHistory<T extends string, D extends object> {
    event_type: T;
    at: DateTime;
    by: User;
    data: D;
}

export interface PersonHistory {
    type: "ticket_person_history";
    id: number;
    startDate: DateTime;
    endDate: null | DateTime;
    person: Person;
    removedBy: null;
    createdBy: User | null;
    createdAt: DateTime | null;
}

export interface MotiveHistory {
    type: "ticket_motive_history";
    id: number;
    startDate: null;
    endDate: null | DateTime;
    motive: Motive;
    createdBy: User | null;
    createdAt: DateTime | null;
}

export interface Comment {
    type: "ticket_comment";
    id: number;
    content: string;
    createdBy: User | null;
    createdAt: DateTime | null;
    updatedBy: User | null;
    updatedAt: DateTime | null;
}

export interface AddresseeHistory {
    type: "ticket_addressee_history";
    id: number;
    startDate: DateTime | null;
    addressee: UserGroupOrUser;
    endDate: DateTime | null;
    removedBy: User | null;
    createdBy: User | null;
    createdAt: DateTime | null;
    updatedBy: User | null;
    updatedAt: DateTime | null;
}

export interface AddresseeState {
    addressees: UserGroupOrUser[];
}

export interface PersonsState {
    persons: Person[];
}

export interface CreateTicketState {}

//interface AddPersonEvent extends TicketHistory<"add_person", PersonHistory> {};
export interface AddCommentEvent
    extends TicketHistory<"add_comment", Comment> {}
export interface SetMotiveEvent
    extends TicketHistory<"set_motive", MotiveHistory> {}
//interface AddAddressee extends TicketHistory<"add_addressee", AddresseeHistory> {};
//interface RemoveAddressee extends TicketHistory<"remove_addressee", AddresseeHistory> {};
export interface AddresseesStateEvent
    extends TicketHistory<"addressees_state", AddresseeState> {}
export interface CreateTicketEvent
    extends TicketHistory<"create_ticket", CreateTicketState> {}
export interface PersonStateEvent
    extends TicketHistory<"persons_state", PersonsState> {}

export type TicketHistoryLine =
    /* AddPersonEvent */
    | CreateTicketEvent
    | AddCommentEvent
    | SetMotiveEvent /*AddAddressee | RemoveAddressee | */
    | AddresseesStateEvent
    | PersonStateEvent;

export interface Ticket {
    type: "ticket_ticket";
    id: number;
    externalRef: string;
    currentAddressees: UserGroupOrUser[];
    currentPersons: Person[];
    currentMotive: null | Motive;
    history: TicketHistoryLine[];
    createdAt: DateTime | null;
    updatedBy: User | null;
}
