import {
    fetchResults,
    makeFetch,
} from "../../../../../../../../ChillMainBundle/Resources/public/lib/api/apiMethods";

import { Module } from "vuex";
import { RootState } from "..";

import {
    User,
    UserGroup,
    UserGroupOrUser,
} from "../../../../../../../../ChillMainBundle/Resources/public/types";

export interface State {
    userGroups: UserGroup[];
    users: User[];
}

export const moduleAddressee: Module<State, RootState> = {
    state: () => ({
        userGroups: [] as UserGroup[],
        users: [] as User[],
    }),
    getters: {
        getUserGroups(state) {
            return state.userGroups;
        },
        getUsers(state) {
            return state.users;
        },
    },
    mutations: {
        setUserGroups(state, userGroups) {
            state.userGroups = userGroups;
        },
        setUsers(state, users) {
            state.users = users;
        },
    },
    actions: {
        fetchUserGroups({ commit }) {
            try {
                fetchResults("/api/1.0/main/user-group.json").then(
                    (results) => {
                        commit("setUserGroups", results);
                    },
                );
            } catch (e: any) {
                throw e.name;
            }
        },
        fetchUsers({ commit }) {
            try {
                fetchResults("/api/1.0/main/user.json").then((results) => {
                    commit("setUsers", results);
                });
            } catch (e: any) {
                throw e.name;
            }
        },

        async setAdressees(
            { commit },
            datas: { ticketId: number; addressees: UserGroupOrUser[] },
        ) {
            const { ticketId, addressees } = datas;
            try {
                const result = await makeFetch(
                    "POST",
                    `/api/1.0/ticket/${ticketId}/addressees/set`,
                    {
                        addressees: addressees.map((addressee) => {
                            return { id: addressee.id, type: addressee.type };
                        }),
                    },
                );
                commit("setTicket", result);
            } catch (e: any) {
                throw e.name;
            }
        },
    },
};
