import {
    fetchResults,
    makeFetch,
} from "../../../../../../../../ChillMainBundle/Resources/public/lib/api/apiMethods";

import { Module } from "vuex";
import { RootState } from "..";

import { Motive } from "../../../../types";

export interface State {
    motives: Motive[];
}

export const moduleMotive: Module<State, RootState> = {
    state: () => ({
        motives: [] as Motive[],
    }),
    getters: {
        getMotives(state) {
            return state.motives;
        },
    },
    mutations: {
        setMotives(state, motives) {
            state.motives = motives;
        },
    },
    actions: {
        async fetchMotives({ commit }) {
            try {
                const results = (await fetchResults(
                    "/api/1.0/ticket/motive.json",
                )) as Motive[];
                commit("setMotives", results);
            } catch (e: any) {
                throw e.name;
            }
        },

        async createMotive(
            { commit },
            datas: { ticketId: number; motive: Motive },
        ) {
            const { ticketId, motive } = datas;
            try {
                const result = await makeFetch(
                    "POST",
                    `/api/1.0/ticket/${ticketId}/motive/set`,
                    {
                        motive: {
                            id: motive.id,
                            type: motive.type,
                        },
                    },
                );
                commit("setTicket", result);
            } catch (e: any) {
                throw e.name;
            }
        },
    },
};
