<?php

declare(strict_types=1);

/*
 * Chill is a software for social workers
 *
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace Chill\TicketBundle\Tests\Action\Ticket\Handler;

use Chill\MainBundle\Phonenumber\PhonenumberHelper;
use Chill\PersonBundle\Entity\Person;
use Chill\PersonBundle\Repository\PersonACLAwareRepositoryInterface;
use Chill\TicketBundle\Action\Ticket\AssociateByPhonenumberCommand;
use Chill\TicketBundle\Action\Ticket\Handler\AssociateByPhonenumberCommandHandler;
use Chill\TicketBundle\Entity\Ticket;
use Doctrine\ORM\EntityManagerInterface;
use PHPUnit\Framework\TestCase;
use Prophecy\Argument;
use Prophecy\PhpUnit\ProphecyTrait;
use Psr\Log\NullLogger;
use Symfony\Component\Cache\Adapter\ArrayAdapter;
use Symfony\Component\Clock\MockClock;
use Symfony\Component\DependencyInjection\ParameterBag\ParameterBag;

/**
 * @internal
 *
 * @coversNothing
 */
class AssociateByPhonenumberCommandHandlerTest extends TestCase
{
    use ProphecyTrait;

    private function getHandler(
        PersonACLAwareRepositoryInterface $personACLAwareRepository,
    ): AssociateByPhonenumberCommandHandler {
        $entityManager = $this->prophesize(EntityManagerInterface::class);
        $phonenumberHelper = new PhonenumberHelper(
            new ArrayAdapter(),
            new ParameterBag([
                'chill_main.phone_helper' => [
                    'default_carrier_code' => 'BE',
                ],
            ]),
            new NullLogger()
        );

        return new AssociateByPhonenumberCommandHandler(
            $personACLAwareRepository,
            $phonenumberHelper,
            new MockClock(),
            $entityManager->reveal()
        );
    }

    public function testHandleWithPersonFoundByPhonenumber(): void
    {
        $person = new Person();

        $personAclAwareRepository = $this->prophesize(PersonACLAwareRepositoryInterface::class);
        $personAclAwareRepository->findByPhone(Argument::any())->willReturn([$person]);

        $handler = $this->getHandler($personAclAwareRepository->reveal());

        $ticket = new Ticket();
        $handler($ticket, new AssociateByPhonenumberCommand('+3281136917'));

        self::assertSame($person, $ticket->getPersons()[0]);
    }
}
