<?php

declare(strict_types=1);

/*
 * Chill is a software for social workers
 *
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace Chill\TicketBundle\Tests\Action\Ticket\Handler;

use Chill\TicketBundle\Action\Ticket\Handler\ReplaceMotiveCommandHandler;
use Chill\TicketBundle\Action\Ticket\ReplaceMotiveCommand;
use Chill\TicketBundle\Entity\Motive;
use Chill\TicketBundle\Entity\MotiveHistory;
use Chill\TicketBundle\Entity\Ticket;
use Doctrine\ORM\EntityManagerInterface;
use Prophecy\Argument;
use Prophecy\PhpUnit\ProphecyTrait;
use Symfony\Bundle\FrameworkBundle\Test\KernelTestCase;
use Symfony\Component\Clock\MockClock;

/**
 * @internal
 *
 * @coversNothing
 */
final class ReplaceMotiveCommandHandlerTest extends KernelTestCase
{
    use ProphecyTrait;

    private function buildHandler(
        EntityManagerInterface $entityManager,
    ): ReplaceMotiveCommandHandler {
        $clock = new MockClock();

        return new ReplaceMotiveCommandHandler($clock, $entityManager);
    }

    public function testHandleOnTicketWithoutMotive(): void
    {
        $motive = new Motive();
        $ticket = new Ticket();

        $entityManager = $this->prophesize(EntityManagerInterface::class);
        $entityManager->persist(Argument::that(static function ($arg) use ($motive): bool {
            if (!$arg instanceof MotiveHistory) {
                return false;
            }

            return $arg->getMotive() === $motive;
        }))->shouldBeCalled();

        $handler = $this->buildHandler($entityManager->reveal());

        $handler->handle($ticket, new ReplaceMotiveCommand($motive));

        self::assertSame($motive, $ticket->getMotive());
    }

    public function testHandleReplaceMotiveOnTicketWithExistingMotive(): void
    {
        $motive = new Motive();
        $ticket = new Ticket();
        $history = new MotiveHistory(new Motive(), $ticket);

        $entityManager = $this->prophesize(EntityManagerInterface::class);
        $entityManager->persist(Argument::that(static function ($arg) use ($motive): bool {
            if (!$arg instanceof MotiveHistory) {
                return false;
            }

            return $arg->getMotive() === $motive;
        }))->shouldBeCalled();

        $handler = $this->buildHandler($entityManager->reveal());

        $handler->handle($ticket, new ReplaceMotiveCommand($motive));

        self::assertSame($motive, $ticket->getMotive());
        self::assertCount(2, $ticket->getMotiveHistories());
    }

    public function testHandleReplaceMotiveOnTicketWithSameMotive(): void
    {
        $motive = new Motive();
        $ticket = new Ticket();
        $history = new MotiveHistory($motive, $ticket);

        $entityManager = $this->prophesize(EntityManagerInterface::class);
        $entityManager->persist(Argument::that(static function ($arg) use ($motive): bool {
            if (!$arg instanceof MotiveHistory) {
                return false;
            }

            return $arg->getMotive() === $motive;
        }))->shouldNotBeCalled();

        $handler = $this->buildHandler($entityManager->reveal());

        $handler->handle($ticket, new ReplaceMotiveCommand($motive));

        self::assertSame($motive, $ticket->getMotive());
        self::assertCount(1, $ticket->getMotiveHistories());
    }
}
